#!/usr/bin/env python3

import argparse
import os
import platform
import re
import sys
import signal
from PyQt6.QtWidgets import (QApplication, QMainWindow, QDialogButtonBox,
                               QDialog, QTreeWidgetItem, QListWidgetItem,
                               QListView, QButtonGroup)
from PyQt6.QtGui import QPixmap
from PyQt6.QtCore import Qt, QTranslator, QLocale, QLibraryInfo
from shutil import copy2
from sway_input_config.utils import (list_inputs_by_type, get_data_dir,
                                     get_sway_version,
                                     get_config_home,
                                     load_json, save_json,
                                     save_list_to_text_file,
                                     load_text_file, reload_sway_config)
from sway_input_config.ui_mainwindow import Ui_MainWindow
from sway_input_config.ui_about import Ui_about
from sway_input_config.ui_selectlayout import Ui_SelectKeyboardLayoutDialog
from sway_input_config.ui_error_message import Ui_ErrorMessage

app_version = "1.4.1"

if os.getenv("SWAYSOCK"):
    sway_version = get_sway_version()
    data_dir = ""
    config_home = get_config_home()
    sway_config = os.path.join(config_home, "sway", "config")

dir_name = os.path.dirname(__file__)
default_settings = os.path.join(dir_name, "data/defaults.json")

os_name = platform.system()

if os_name == "Linux":
    XKB_BASE_LIST = load_text_file("/usr/share/X11/xkb/rules/base.lst").splitlines()
elif os_name == "FreeBSD":
    XKB_BASE_LIST = load_text_file("/usr/local/share/X11/xkb/rules/base.lst").splitlines()

models = []
layouts = []
variants = []
options = []

for line in XKB_BASE_LIST:
    if not line:
        continue
    match = re.match(r'^!\s*(\w+)\s*$', line)
    if match:
        category = match.group(1)
    else:
        match = re.match(r'^\s*(\w+:\w+|\w+-\w+|\w+)\s*(.*)$', line)
        if not match:
            continue
        key = match.group(1)
        value = match.group(2)
        if category == 'model':
            models.append((key, value))
        elif category == 'layout':
            layouts.append((key, value))
        elif category == 'variant':
            variants.append((key, value))
        elif category == 'option':
            options.append((key, value))

# Buttons array used for on_button_down scroll method
scroll_buttons = {
    "Disabled": "disable",
    "Left button": "BTN_LEFT",
    "Right button": "BTN_RIGHT",
    "Middle button": "BTN_MIDDLE",
    "Side button": "BTN_SIDE",
    "Extra button": "BTN_EXTRA"
    }


class ErrorMessage(QDialog):
    def __init__(self):
        super(ErrorMessage, self).__init__()
        self.ui = Ui_ErrorMessage()
        self.ui.setupUi(self)

        self.btnOk = self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok)
        self.btnOk.clicked.connect(self.on_clicked_ok)

    def on_clicked_ok(self):
        self.close()


class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        # Dialog buttons
        self.ui.buttonBox.rejected.connect(self.cancel)
        self.btnApply = self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Apply)
        self.btnApply.clicked.connect(self.on_clicked_apply)
        self.btnReset = self.ui.buttonBox.button(QDialogButtonBox.StandardButton.RestoreDefaults)
        self.btnReset.clicked.connect(self.on_clicked_reset)
        self.ui.buttonBox.helpRequested.connect(self.on_clicked_about)

        # Keyboard Settings #

        # Use this settings
        if settings["keyboard-use-settings"] == "true":
            self.ui.KeyBoardUseSettings.setChecked(True)
        self.ui.KeyBoardUseSettings.toggled.connect(self.keyboard_use_settings)

        # Keyboard layout
        for key, value in layouts:
            if key in settings["keyboard-layout"]:
                self.layout_item = QTreeWidgetItem(self.ui.layouts)
                self.layout_item.setData(0, Qt.ItemDataRole.DisplayRole, value)
                self.layout_item.setData(0, Qt.ItemDataRole.UserRole, key)
                self.ui.layouts.addTopLevelItem(self.layout_item)
                for key, values in variants:
                    value = values.split(":")[0]
                    description = values.split(":")[1]
                    if value in self.layout_item.data(0, Qt.ItemDataRole.UserRole):
                        # Workaround to prevent custom layout from using variants for English(US)
                        if "custom" not in self.layout_item.data(0, Qt.ItemDataRole.UserRole):
                            if key in settings["keyboard-variant"]:
                                self.layout_item.setData(1, Qt.ItemDataRole.DisplayRole, description)
                                self.layout_item.setData(1, Qt.ItemDataRole.UserRole, key)

        self.ui.addBtn.clicked.connect(self.on_add_keyboard_layout)
        self.ui.rmBtn.clicked.connect(self.on_remove_layout)
        self.ui.upBtn.clicked.connect(self.on_move_up)
        self.ui.downBtn.clicked.connect(self.on_move_down)

        # Keyboard ID #
        keyboards = list_inputs_by_type(input_type="keyboard")
        kbd_view = QListView(self.ui.kbdID)
        self.ui.kbdID.setView(kbd_view)
        self.ui.kbdID.addItem("")
        for item in keyboards:
            self.ui.kbdID.addItem(item)
        kbd_view.setTextElideMode(Qt.TextElideMode.ElideNone)
        kbd_view.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        self.ui.kbdID.setCurrentText(settings["keyboard-identifier"])
        self.ui.kbdID.activated.connect(self.set_kbd_identifier)

        # Keyboard model option
        model_view = QListView(self.ui.kbdModel)
        self.ui.kbdModel.setView(model_view)
        model_view.setTextElideMode(Qt.TextElideMode.ElideNone)
        model_view.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        for key, value in models:
            self.ui.kbdModel.addItem(value, key)
            if key == settings["keyboard-model"]:
                self.ui.kbdModel.setCurrentText(value)

        self.ui.kbdModel.activated.connect(self.set_model)

        # Keyboard shortcut option
        shortcut_view = QListView(self.ui.shortcutName)
        self.ui.shortcutName.setView(shortcut_view)
        self.ui.shortcutName.addItem("")
        for key, value in options:
            if "grp:" in key:
                self.ui.shortcutName.addItem(value, key)
            if key == settings["keyboard-shortcut"]:
                self.ui.shortcutName.setCurrentText(value)
        shortcut_view.setTextElideMode(Qt.TextElideMode.ElideNone)
        shortcut_view.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        self.ui.shortcutName.activated.connect(self.set_shortcut)

        # Repeat delay
        self.ui.repeatDelaySlider.setValue(settings["keyboard-repeat-delay"])
        self.ui.repeatDelay.setValue(self.ui.repeatDelaySlider.value())
        self.ui.repeatDelay.valueChanged.connect(self.ui.repeatDelaySlider.setValue)
        self.ui.repeatDelaySlider.sliderMoved.connect(self.ui.repeatDelay.setValue)
        self.ui.repeatDelaySlider.valueChanged.connect(self.on_repeat_delay_value_changed)

        # Repeat rate
        self.ui.repeatRateSlider.setValue(settings["keyboard-repeat-rate"])
        self.ui.repeatRate.setValue(self.ui.repeatRateSlider.value())
        self.ui.repeatRate.valueChanged.connect(self.ui.repeatRateSlider.setValue)
        self.ui.repeatRateSlider.sliderMoved.connect(self.ui.repeatRate.setValue)
        self.ui.repeatRateSlider.valueChanged.connect(self.on_repeat_rate_value_changed)

        # Enable Caps Lock option
        if settings["keyboard-capslock"] == "enabled":
            self.ui.caps_lock.setChecked(True)
        self.ui.caps_lock.toggled.connect(self.on_caps_lock_checked)

        # Enable Num Lock option
        if settings["keyboard-numlock"] == "enabled":
            self.ui.num_lock.setChecked(True)
        self.ui.num_lock.toggled.connect(self.on_num_lock_checked)

        # Mouse Settings #

        # Use this settings
        if settings["pointer-use-settings"] == "true":
            self.ui.PointerUseSettings.setChecked(True)
        self.ui.PointerUseSettings.toggled.connect(self.pointer_use_settings)

        # Pointer ID #
        pointers = list_inputs_by_type(input_type="pointer")
        pointers_view = QListView(self.ui.pointerID)
        self.ui.pointerID.setView(pointers_view)
        self.ui.pointerID.addItem("")
        for item in pointers:
            self.ui.pointerID.addItem(item)
        pointers_view.setTextElideMode(Qt.TextElideMode.ElideNone)
        pointers_view.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        self.ui.pointerID.setCurrentText(settings["pointer-identifier"])
        self.ui.pointerID.activated.connect(self.set_pointer_identifier)

        # Left handed mode
        if settings["pointer-left-handed"] == "true":
            self.ui.pointerLeftHanded.setChecked(True)
        self.ui.pointerLeftHanded.toggled.connect(self.on_pointer_left_handed_checked)

        # Middle click emulation
        if settings["pointer-middle-emulation"] == "enabled":
            self.ui.pointerMiddle.setChecked(True)
        self.ui.pointerMiddle.toggled.connect(self.on_middle_checked)

        # Pointer speed
        pointerAccelValue = float(settings["pointer-pointer-accel"]) * 10
        self.ui.pointerAccel.setValue(int(pointerAccelValue))
        self.ui.pointerAccel.valueChanged.connect(self.on_accel_value_changed)

        # Acceleration profile
        self.pointerAccelButtonGroup = QButtonGroup()
        self.pointerAccelButtonGroup.addButton(self.ui.pointerFlat)
        self.pointerAccelButtonGroup.addButton(self.ui.pointerAdaptive)
        if settings["pointer-accel-profile"] == "flat":
            self.ui.pointerFlat.setChecked(True)
        else:
            self.ui.pointerAdaptive.setChecked(True)
        self.ui.pointerFlat.clicked.connect(self.on_accel_profile_changed)
        self.ui.pointerAdaptive.clicked.connect(self.on_accel_profile_changed)

        # Rotation angle (0.0 to 360.0) since Sway 1.9
        if sway_version >= "1.9":
            pointerRotationValue = float(settings["pointer-rotation-angle"])
            self.ui.pointerRotationAngleSlider.setValue(int(pointerRotationValue))
            self.ui.pointerRotationAngle.setValue(self.ui.pointerRotationAngleSlider.value())
            self.ui.pointerRotationAngleSlider.sliderMoved.connect(self.ui.pointerRotationAngle.setValue)
            self.ui.pointerRotationAngleSlider.valueChanged.connect(self.on_pointer_rotation_angle_value_changed)
        else:
            self.ui.pointerRotationAngleSlider.setDisabled(True)
            self.ui.pointerRotationAngle.setDisabled(True)
            self.ui.pointerRotationLabel.setDisabled(True)

        # Scrolling
        if settings["pointer-natural-scroll"] == "true":
            self.ui.pointerNatScroll.setChecked(True)
        self.ui.pointerNatScroll.toggled.connect(self.on_pointer_nat_scroll_checked)

        # Scrolling speed:
        pointerFactorValue = float(settings["pointer-scroll-factor"]) * 10
        self.ui.pointerScrollFactor.setValue(int(pointerFactorValue))
        self.ui.pointerScrollFactor.valueChanged.connect(self.on_pointer_scroll_value_changed)

        # Scrolling button (trackpoints only)
        for key, value in scroll_buttons.items():
            self.ui.scrollButtonList.addItem(key, value)
            if value == settings["pointer-scroll-button"]:
                self.ui.scrollButtonList.setCurrentText(key)
        self.ui.scrollButtonList.activated.connect(self.on_scroll_button_checked)

        # Scrolling button lock (since Sway 1.9)
        if sway_version >= "1.9":
            self.ui.scrollButtonLock.setEnabled(True)
            if settings["pointer-scroll-button-lock"] == "enabled":
                self.ui.scrollButtonLock.setChecked(True)
            self.ui.scrollButtonLock.clicked.connect(self.on_scroll_button_lock_checked)

        # Touchapd Settings #

        # Use this settings
        if settings["touchpad-use-settings"] == "true":
            self.ui.TouchPadUseSettings.setChecked(True)
        self.ui.TouchPadUseSettings.toggled.connect(self.touchpad_use_settings)

        # Touchpad ID #
        touchpads = list_inputs_by_type(input_type="touchpad")
        touchpads_view = QListView(self.ui.touchpadID)
        self.ui.touchpadID.setView(touchpads_view)
        if not touchpads:
            self.ui.touchpadID.addItem("Not found")
            self.ui.tabWidget.widget(2).setEnabled(False)
        else:
            self.ui.touchpadID.addItem("")
            for item in touchpads:
                self.ui.touchpadID.addItem(item)
                touchpads_view.setTextElideMode(Qt.TextElideMode.ElideNone)
                touchpads_view.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
                self.ui.touchpadID.setCurrentText(settings["touchpad-identifier"])
                self.ui.touchpadID.activated.connect(self.set_touchpad_identifier)

        # Touchapd send events mode:
        self.eventsButtonGroup = QButtonGroup()
        self.eventsButtonGroup.addButton(self.ui.touchEventsEnabled)
        self.eventsButtonGroup.addButton(self.ui.touchEventsDisabled)
        self.eventsButtonGroup.addButton(self.ui.touchEventsOnExternalMouse)
        if settings["touchpad-events"] == "disabled":
            self.ui.touchEventsDisabled.setChecked(True)
        elif settings["touchpad-events"] == "disabled_on_external_mouse":
            self.ui.touchEventsOnExternalMouse.setChecked(True)
        else:
            self.ui.touchEventsEnabled.setChecked(True)
        self.ui.touchEventsEnabled.clicked.connect(self.on_touchpad_events_mode_changed)
        self.ui.touchEventsDisabled.clicked.connect(self.on_touchpad_events_mode_changed)
        self.ui.touchEventsOnExternalMouse.clicked.connect(self.on_touchpad_events_mode_changed)

        # Disable while typing
        if settings["touchpad-dwt"] == "enabled":
            self.ui.DWT.setChecked(True)
        self.ui.DWT.toggled.connect(self.on_dwt_checked)

        # Disable while trackpointing
        if sway_version >= "1.8":
            if settings["touchpad-dwtp"] == "enabled":
                self.ui.DWTP.setChecked(True)
            self.ui.DWTP.toggled.connect(self.on_dwtp_checked)
        else:
            self.ui.DWTP.setEnabled(False)

        # Left handed mode
        if settings["touchpad-left-handed"] == "enabled":
            self.ui.touchLeftHanded.setChecked(True)
        self.ui.touchLeftHanded.toggled.connect(self.on_touch_left_handed_checked)

        # Middle click emulation
        if settings["touchpad-middle-emulation"] == "enabled":
            self.ui.touchMiddle.setChecked(True)
        self.ui.touchMiddle.toggled.connect(self.on_touch_middle_emu_checked)

        # Pointer speed:
        touchAccelValue = float(settings["touchpad-pointer-accel"]) * 10
        self.ui.touchAccel.setValue(int(touchAccelValue))
        self.ui.touchAccel.valueChanged.connect(self.on_touch_accel_value_changed)

        # Acceleration profile
        self.touchAccelButtonGroup = QButtonGroup()
        self.touchAccelButtonGroup.addButton(self.ui.touchFlat)
        self.touchAccelButtonGroup.addButton(self.ui.touchAdaptive)
        if settings["touchpad-accel-profile"] == "flat":
            self.ui.touchFlat.setChecked(True)
        else:
            self.ui.touchAdaptive.setChecked(True)
        self.ui.touchFlat.clicked.connect(self.on_touch_accel_profile_changed)
        self.ui.touchAdaptive.clicked.connect(self.on_touch_accel_profile_changed)

        # Rotation angle (0.0 to 360.0) since Sway 1.9
        if sway_version >= "1.9":
            touchRotationValue = float(settings["touchpad-rotation-angle"])
            self.ui.touchRotationAngleSlider.setValue(int(touchRotationValue))
            self.ui.touchRotationAngle.setValue(self.ui.touchRotationAngleSlider.value())
            self.ui.touchRotationAngleSlider.sliderMoved.connect(self.ui.touchRotationAngle.setValue)
            self.ui.touchRotationAngleSlider.valueChanged.connect(self.on_touch_rotation_angle_value_changed)
        else:
            self.ui.touchRotationAngleSlider.setDisabled(True)
            self.ui.touchRotationAngle.setDisabled(True)
            self.ui.touchpadRotationLabel.setDisabled(True)

        # Tap-to-click
        if settings["touchpad-tap"] == "enabled":
            self.ui.tap_click.setChecked(True)
        self.ui.tap_click.toggled.connect(self.on_tap_click_checked)

        # Tap-and-drag
        if settings["touchpad-tap"] == "enabled":
            self.ui.drag.setEnabled(True)
        else:
            self.ui.drag.setEnabled(False)
        if settings["touchpad-drag"] == "enabled":
            self.ui.drag.setChecked(True)
        self.ui.drag.toggled.connect(self.on_tapdrag_checked)

        # Tap-and-drag lock
        if settings["touchpad-tap"] == "enabled":
            self.ui.drag_lock.setEnabled(True)
        else:
            self.ui.drag_lock.setEnabled(False)
        if settings["touchpad-drag-lock"] == "enabled":
            self.ui.drag_lock.setChecked(True)
        self.ui.drag_lock.toggled.connect(self.on_draglock_checked)

        # Tap button mapping:
        self.mappingButtonGroup = QButtonGroup()
        self.mappingButtonGroup.addButton(self.ui.lmr)
        self.mappingButtonGroup.addButton(self.ui.lrm)
        if settings["touchpad-tap"] == "enabled":
            self.ui.lrm.setEnabled(True)
            self.ui.lmr.setEnabled(True)
        else:
            self.ui.lrm.setEnabled(False)
            self.ui.lmr.setEnabled(False)
        if settings["touchpad-tap-button-map"] == "lrm":
            self.ui.lrm.setChecked(True)
        else:
            self.ui.lmr.setChecked(True)
        self.ui.lrm.clicked.connect(self.on_multi_tap_checked)
        self.ui.lmr.clicked.connect(self.on_multi_tap_checked)

        # Click method
        self.clickMethodButtonGroup = QButtonGroup()
        self.clickMethodButtonGroup.addButton(self.ui.btn_BtnArea)
        self.clickMethodButtonGroup.addButton(self.ui.btn_ClickFinger)
        if settings["touchpad-click-method"] == "button_areas":
            self.ui.btn_BtnArea.setChecked(True)
        elif settings["touchpad-click-method"] == "clickfinger":
            self.ui.btn_ClickFinger.setChecked(True)
        self.ui.btn_BtnArea.clicked.connect(self.on_click_method_checked)
        self.ui.btn_ClickFinger.clicked.connect(self.on_click_method_checked)

        # Scrolling method
        self.scrollingButtonGroup = QButtonGroup()
        self.scrollingButtonGroup.addButton(self.ui.touchScrollNoScroll)
        self.scrollingButtonGroup.addButton(self.ui.touchScrollTwoFingers)
        self.scrollingButtonGroup.addButton(self.ui.touchScrollEdges)
        if settings["touchpad-scroll-method"] == "two_finger":
            self.ui.touchScrollTwoFingers.setChecked(True)
        elif settings["touchpad-scroll-method"] == "edge":
            self.ui.touchScrollEdges.setChecked(True)
        else:
            self.ui.touchScrollNoScroll.setChecked(True)

        self.ui.touchScrollNoScroll.clicked.connect(self.on_scroll_method_checked)
        self.ui.touchScrollTwoFingers.clicked.connect(self.on_scroll_method_checked)
        self.ui.touchScrollEdges.clicked.connect(self.on_scroll_method_checked)

        # Natural (inverted) scrolling:
        if settings["touchpad-natural-scroll"] == "enabled":
            self.ui.touchNatScroll.setChecked(True)
        self.ui.touchNatScroll.toggled.connect(self.on_touch_nat_scroll_checked)

        # Scrolling speed
        touchFactorValue = float(settings["touchpad-scroll-factor"]) * 10
        self.ui.touchScrollFactor.setValue(int(touchFactorValue))
        self.ui.touchScrollFactor.valueChanged.connect(self.on_touch_scroll_value_changed)

        # Tablet Settings #

        # Use this settings
        if settings["tablet-use-settings"] == "true":
            self.ui.TabletUseSettings.setChecked(True)
        self.ui.TabletUseSettings.clicked.connect(self.tablet_use_settings)

        # Tablet ID
        tablet_tools = list_inputs_by_type(input_type="tablet_tool")
        tablet_view = QListView(self.ui.tabletID)
        self.ui.tabletID.setView(tablet_view)
        if not tablet_tools:
            self.ui.tabletID.addItem("Not found")
            self.ui.tabWidget.widget(3).setEnabled(False)
        else:
            self.ui.tabletID.addItem("")
            for item in tablet_tools:
                self.ui.tabletID.addItem(item)
                tablet_view.setTextElideMode(Qt.TextElideMode.ElideNone)
                tablet_view.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
                self.ui.tabletID.setCurrentText(settings["tablet-identifier"])

        # Left handed mode (invert tablet's input)
        if settings["tablet-left-handed"] == "enabled":
            self.ui.tabletLeftHanded.setChecked(True)
        self.ui.tabletLeftHanded.clicked.connect(self.on_tablet_left_handed_checked)

        # Tablet tool mode ("pen", "eraser", "brush", "pencil", "airbrush", and the wildcard *, which matches all tools)
        mode = ["*", "pen", "eraser", "brush", "pencil", "airbrush"]
        for item in mode:
            self.ui.toolModeList.addItem(item)
        self.ui.toolModeList.setCurrentText(settings["tablet-tool-mode"][0])
        self.ui.toolModeList.activated.connect(self.on_tablet_set_tool_mode)

        # Tablet tool movement (absolute or relative)
        self.toolMoveButtonGroup = QButtonGroup()
        self.toolMoveButtonGroup.addButton(self.ui.toolMoveAbsolute)
        self.toolMoveButtonGroup.addButton(self.ui.toolMoveRelative)
        if settings["tablet-tool-mode"][1] == "absolute":
            self.ui.toolMoveAbsolute.setChecked(True)
        else:
            self.ui.toolMoveRelative.setChecked(True)
        self.ui.toolMoveAbsolute.clicked.connect(self.on_tablet_set_tool_mode)
        self.ui.toolMoveRelative.clicked.connect(self.on_tablet_set_tool_mode)


    def keyboard_use_settings(self):
        if self.ui.KeyBoardUseSettings.isChecked() is True:
            settings["keyboard-use-settings"] = "true"
        else:
            settings["keyboard-use-settings"] = "false"

    def on_clicked_reset(self):
        defaults = load_json(default_settings)
        self.ui.layouts.clear()
        for key, values in layouts:
            if key in defaults["keyboard-layout"]:
                self.layout_item = QTreeWidgetItem(self.ui.layouts)
                self.layout_item.setData(0, Qt.ItemDataRole.DisplayRole, values)
                self.layout_item.setData(0, Qt.ItemDataRole.UserRole, key)
                self.ui.layouts.addTopLevelItem(self.layout_item)
                for key, values in variants:
                    value = values.split(":")[0]
                    description = values.split(":")[1]
                    if value in self.layout_item.data(0, Qt.ItemDataRole.UserRole):
                        # Workaround to prevent custom layout from using variants for English(US)
                        if "custom" not in self.layout_item.data(0, Qt.ItemDataRole.UserRole):
                            if key in defaults["keyboard-variant"]:
                                self.layout_item.setData(1, Qt.ItemDataRole.DisplayRole, description)
                                self.layout_item.setData(1, Qt.ItemDataRole.UserRole, key)
        for key, value in models:
            if key == defaults["keyboard-model"]:
                self.ui.kbdModel.setCurrentText(value)
        self.ui.shortcutName.setCurrentText(defaults["keyboard-shortcut"])
        self.ui.kbdID.setCurrentText(defaults["keyboard-identifier"])
        self.ui.repeatDelaySlider.setValue(defaults["keyboard-repeat-delay"])
        self.ui.repeatDelay.setValue(self.ui.repeatDelaySlider.value())
        self.ui.repeatRateSlider.setValue(defaults["keyboard-repeat-rate"])
        self.ui.repeatRate.setValue(self.ui.repeatRateSlider.value())
        self.ui.caps_lock.setChecked(False)
        self.ui.num_lock.setChecked(False)

        self.ui.pointerID.setCurrentText(defaults["pointer-identifier"])
        self.ui.pointerLeftHanded.setChecked(False)
        self.ui.pointerMiddle.setChecked(False)

        pointerAccelValue = float(defaults["pointer-pointer-accel"]) * 10
        self.ui.pointerAccel.setValue(int(pointerAccelValue))

        pointerRotationAngleValue = float(defaults["pointer-rotation-angle"])
        self.ui.pointerRotationAngleSlider.setValue(int(pointerRotationAngleValue))
        self.ui.pointerRotationAngle.setValue(self.ui.pointerRotationAngleSlider.value())

        for key, value in scroll_buttons.items():
            if value == defaults["pointer-scroll-button"]:
                self.ui.scrollButtonList.setCurrentText(key)

        self.ui.pointerFlat.setChecked(True)
        self.ui.pointerNatScroll.setChecked(False)
        self.ui.scrollButtonLock.setChecked(False)
        self.ui.scrollButtonList.setCurrentText(defaults["pointer-scroll-button"])

        pointerFactorValue = float(defaults["pointer-scroll-factor"]) * 10
        self.ui.pointerScrollFactor.setValue(int(pointerFactorValue))

        touchpads = list_inputs_by_type(input_type="touchpad")
        if touchpads:
            self.ui.touchpadID.setCurrentText(defaults["touchpad-identifier"])
            self.ui.touchEventsEnabled.setChecked(True)
            self.ui.DWT.setChecked(True)
            self.ui.DWTP.setChecked(False)
            self.ui.touchLeftHanded.setChecked(False)
            self.ui.touchMiddle.setChecked(True)

            touchAccelValue = float(defaults["touchpad-pointer-accel"]) * 10
            self.ui.touchAccel.setValue(int(touchAccelValue))

            touchpadRotationAngleValue = float(defaults["touchpad-rotation-angle"])
            self.ui.touchRotationAngleSlider.setValue(int(touchpadRotationAngleValue))
            self.ui.touchRotationAngle.setValue(self.ui.touchRotationAngleSlider.value())

            self.ui.touchFlat.setChecked(True)
            self.ui.tap_click.setChecked(True)
            self.ui.drag.setEnabled(True)
            self.ui.drag.setChecked(True)
            self.ui.drag_lock.setChecked(False)
            self.ui.lrm.setEnabled(True)
            self.ui.lmr.setEnabled(True)
            self.ui.touchScrollNoScroll.setChecked(True)
            self.ui.btn_BtnArea.setChecked(True)
            self.ui.touchNatScroll.setChecked(False)

            touchFactorValue = float(defaults["touchpad-scroll-factor"]) * 10
            self.ui.touchScrollFactor.setValue(int(touchFactorValue))

        tablet_tools = list_inputs_by_type(input_type="tablet_tool")
        if tablet_tools:
            self.ui.tabletID.setCurrentText(defaults["tablet-identifier"])
            self.ui.tabletLeftHanded.setChecked(False)

            self.ui.toolModeList.setCurrentText(defaults["tablet-tool-mode"][0])
            self.ui.toolMoveAbsolute.setChecked(True)

    def on_add_keyboard_layout(self):
        self.dlg = SelectKeyboardLayout()
        if self.dlg.exec() == 1:
            lay_key = self.dlg.select_layout.layouts.currentItem().data(Qt.ItemDataRole.DisplayRole)
            lay_value = self.dlg.select_layout.layouts.currentItem().data(Qt.ItemDataRole.UserRole)
            var_key = self.dlg.select_layout.variants.currentItem().data(Qt.ItemDataRole.DisplayRole)
            var_value = self.dlg.select_layout.variants.currentItem().data(Qt.ItemDataRole.UserRole)
            self.item = QTreeWidgetItem(self.ui.layouts)
            self.item.setData(0, Qt.ItemDataRole.DisplayRole, lay_key)
            self.item.setData(0, Qt.ItemDataRole.UserRole, lay_value)
            self.item.setData(1, Qt.ItemDataRole.DisplayRole, var_key)
            self.item.setData(1, Qt.ItemDataRole.UserRole, var_value)
            self.ui.layouts.addTopLevelItem(self.item)

    def on_remove_layout(self):
        if self.ui.layouts.topLevelItemCount() > 1:
            item = self.ui.layouts.currentItem()
            pos = self.ui.layouts.indexOfTopLevelItem(item)
            self.ui.layouts.takeTopLevelItem(pos)

    def on_move_up(self):
        item = self.ui.layouts.currentItem()
        pos = self.ui.layouts.indexOfTopLevelItem(item)
        if pos > 0:
            self.ui.layouts.takeTopLevelItem(pos)
            self.ui.layouts.insertTopLevelItem(pos - 1, item)
            self.ui.layouts.setCurrentItem(item)

    def on_move_down(self):
        item = self.ui.layouts.currentItem()
        pos = self.ui.layouts.indexOfTopLevelItem(item)
        if pos < self.ui.layouts.topLevelItemCount() - 1:
            self.ui.layouts.takeTopLevelItem(pos)
            self.ui.layouts.insertTopLevelItem(pos + 1, item)
            self.ui.layouts.setCurrentItem(item)

    def set_keyboard_layout(self):
        n = self.ui.layouts.topLevelItemCount()
        layouts = []
        variants = []
        row = 0
        if n > 0:
            while row < n:
                item = self.ui.layouts.topLevelItem(row)
                if item.data(1, Qt.ItemDataRole.UserRole) is not None:
                    variants.append(item.data(1, Qt.ItemDataRole.UserRole))
                layouts.append(item.data(0, Qt.ItemDataRole.UserRole))
                row += 1
        settings["keyboard-layout"] = layouts
        settings["keyboard-variant"] = variants

    def set_kbd_identifier(self):
        settings["keyboard-identifier"] = self.ui.kbdID.currentText()

    def set_model(self):
        settings["keyboard-model"] = self.ui.kbdModel.currentData()

    def set_shortcut(self):
        settings["keyboard-shortcut"] = self.ui.shortcutName.currentData()

    def on_repeat_delay_value_changed(self):
        settings["keyboard-repeat-delay"] = self.ui.repeatDelay.value()

    def on_repeat_rate_value_changed(self):
        settings["keyboard-repeat-rate"] = self.ui.repeatRate.value()

    def on_caps_lock_checked(self):
        if self.ui.caps_lock.isChecked():
            settings["keyboard-capslock"] = "enabled"
        else:
            settings["keyboard-capslock"] = "disabled"

    def on_num_lock_checked(self):
        if self.ui.num_lock.isChecked():
            settings["keyboard-numlock"] = "enabled"
        else:
            settings["keyboard-numlock"] = "disabled"

    def pointer_use_settings(self):
        if self.ui.PointerUseSettings.isChecked() is True:
            settings["pointer-use-settings"] = "true"
        else:
            settings["pointer-use-settings"] = "false"

    def set_pointer_identifier(self):
        settings["pointer-identifier"] = self.ui.pointerID.currentText()

    def on_pointer_left_handed_checked(self):
        if self.ui.pointerLeftHanded.isChecked() is True:
            settings["pointer-left-handed"] = "true"
        else:
            settings["pointer-left-handed"] = "false"

    def on_middle_checked(self):
        if self.ui.pointerMiddle.isChecked():
            settings["pointer-middle-emulation"] = "enabled"
        else:
            settings["pointer-middle-emulation"] = "disabled"

    def on_accel_value_changed(self):
        settings["pointer-pointer-accel"] = self.ui.pointerAccel.value() / 10

    def on_accel_profile_changed(self):
        if self.ui.pointerFlat.isChecked():
            settings["pointer-accel-profile"] = "flat"
        else:
            settings["pointer-accel-profile"] = "adaptive"

    def on_pointer_rotation_angle_value_changed(self):
        settings["pointer-rotation-angle"] = float(self.ui.pointerRotationAngle.value())

    def on_scroll_button_checked(self):
        settings["pointer-scroll-button"] = self.ui.scrollButtonList.currentData()

    def on_scroll_button_lock_checked(self):
        if self.ui.scrollButtonLock.isChecked() is True:
            settings["pointer-scroll-button-lock"] = "enabled"
        else:
            settings["pointer-scroll-button-lock"] = "disabled"

    def on_pointer_nat_scroll_checked(self):
        if self.ui.pointerNatScroll.isChecked() is True:
            settings["pointer-natural-scroll"] = "true"
        else:
            settings["pointer-natural-scroll"] = "false"

    def on_pointer_scroll_value_changed(self):
        settings["pointer-scroll-factor"] = self.ui.pointerScrollFactor.value() / 10

    def touchpad_use_settings(self):
        if self.ui.TouchPadUseSettings.isChecked() is True:
            settings["touchpad-use-settings"] = "true"
        else:
            settings["touchpad-use-settings"] = "false"

    def set_touchpad_identifier(self):
        settings["touchpad-identifier"] = self.ui.touchpadID.currentText()

    def on_touchpad_events_mode_changed(self):
        if self.ui.touchEventsDisabled.isChecked() is True:
            settings["touchpad-events"] = "disabled"
        elif self.ui.touchEventsOnExternalMouse.isChecked() is True:
            settings["touchpad-events"] = "disabled_on_external_mouse"
        else:
            settings["touchpad-events"] = "enabled"

    def on_dwt_checked(self):
        if self.ui.DWT.isChecked():
            settings["touchpad-dwt"] = "enabled"
        else:
            settings["touchpad-dwt"] = "disabled"

    def on_dwtp_checked(self):
        if self.ui.DWTP.isChecked():
            settings["touchpad-dwtp"] = "enabled"
        else:
            settings["touchpad-dwtp"] = "disabled"

    def on_touch_left_handed_checked(self):
        if self.ui.touchLeftHanded.isChecked():
            settings["touchpad-left-handed"] = "enabled"
        else:
            settings["touchpad-left-handed"] = "disabled"

    def on_touch_middle_emu_checked(self):
        if self.ui.touchMiddle.isChecked():
            settings["touchpad-middle-emulation"] = "enabled"
        else:
            settings["touchpad-middle-emulation"] = "disabled"

    def on_touch_accel_value_changed(self):
        settings["touchpad-pointer-accel"] = self.ui.touchAccel.value() / 10

    def on_touch_accel_profile_changed(self):
        if self.ui.touchFlat.isChecked():
            settings["touchpad-accel-profile"] = "flat"
        else:
            settings["touchpad-accel-profile"] = "adaptive"

    def on_touch_rotation_angle_value_changed(self):
        settings["touchpad-rotation-angle"] = float(self.ui.touchRotationAngle.value())

    def on_tap_click_checked(self):
        if self.ui.tap_click.isChecked():
            settings["touchpad-tap"] = "enabled"
            self.ui.drag.setEnabled(True)
            self.ui.drag_lock.setEnabled(True)
            self.ui.lrm.setEnabled(True)
            self.ui.lmr.setEnabled(True)
        else:
            settings["touchpad-tap"] = "disabled"
            self.ui.drag.setEnabled(False)
            self.ui.drag_lock.setEnabled(False)
            self.ui.lrm.setEnabled(False)
            self.ui.lmr.setEnabled(False)

    def on_tapdrag_checked(self):
        if self.ui.drag.isChecked():
            settings["touchpad-drag"] = "enabled"
        else:
            settings["touchpad-drag"] = "disabled"

    def on_draglock_checked(self):
        if self.ui.drag_lock.isChecked():
            settings["touchpad-drag-lock"] = "enabled"
        else:
            settings["touchpad-drag-lock"] = "disabled"

    def on_multi_tap_checked(self):
        if self.ui.lrm.isChecked():
            settings["touchpad-tap-button-map"] = "lrm"
        else:
            settings["touchpad-tap-button-map"] = "lmr"

    def on_click_method_checked(self):
        if self.ui.btn_ClickFinger.isChecked():
            settings["touchpad-click-method"] = "clickfinger"
        else:
            settings["touchpad-click-method"] = "button_areas"

    def on_scroll_method_checked(self):
        if self.ui.touchScrollTwoFingers.isChecked() is True:
            settings["touchpad-scroll-method"] = "two_finger"
        elif self.ui.touchScrollEdges.isChecked() is True:
            settings["touchpad-scroll-method"] = "edge"
        else:
            settings["touchpad-scroll-method"] = "none"


    def on_touch_nat_scroll_checked(self):
        if self.ui.touchNatScroll.isChecked() is True:
            settings["touchpad-natural-scroll"] = "enabled"
        else:
            settings["touchpad-natural-scroll"] = "disabled"

    def on_touch_scroll_value_changed(self):
        settings["touchpad-scroll-factor"] = self.ui.touchScrollFactor.value() / 10

    def tablet_use_settings(self):
        if self.ui.TabletUseSettings.isChecked() is True:
            settings["tablet-use-settings"] = "true"
        else:
            settings["tablet-use-settings"] = "false"

    def set_tablet_identifier(self):
        settings["tablet-identifier"] = self.ui.tabletID.currentText()

    def on_tablet_left_handed_checked(self):
        if self.ui.tabletLeftHanded.isChecked():
            settings["tablet-left-handed"] = "enabled"
        else:
            settings["tablet-left-handed"] = "disabled"

    def on_tablet_set_tool_mode(self):
        if self.ui.toolMoveAbsolute.isChecked() is True:
            settings["tablet-tool-mode"][1] = "absolute"
        else:
            settings["tablet-tool-mode"][1] = "relative"
        settings["tablet-tool-mode"][0] = self.ui.toolModeList.currentText()

    def on_clicked_about(self):
        self.about = AboutDialog()
        self.about.show()

    def on_clicked_apply(self):
        self.set_keyboard_layout()
        save_to_config()
        f = os.path.join(data_dir, "settings")
        print("Saving {}".format(f))
        save_json(settings, f)
        reload_sway_config()

    def cancel(self):
        self.close()


class AboutDialog(QDialog):
    def __init__(self):
        super().__init__()
        self.aboutDialog = Ui_about()
        self.aboutDialog.setupUi(self)

        self.pixmap = QPixmap(os.path.join(dir_name, "data/logo_sic.png"))
        self.aboutDialog.pixmap.setPixmap(self.pixmap)

        self.aboutDialog.version.setText(self.tr("Version: ") + app_version)

        self.aboutDialog.buttonBox.rejected.connect(self.cancel)

    def cancel(self):
        self.close()


class SelectKeyboardLayout(QDialog):
    def __init__(self):
        super().__init__()
        self.select_layout = Ui_SelectKeyboardLayoutDialog()
        self.select_layout.setupUi(self)

        none_item = QListWidgetItem()
        none_item.setData(Qt.ItemDataRole.UserRole, "")
        none_item.setData(Qt.ItemDataRole.DisplayRole, "")
        self.select_layout.variants.addItem(none_item)

        for key, value in layouts:
            item = QListWidgetItem(value)
            item.setData(Qt.ItemDataRole.UserRole, key)
            item.setData(Qt.ItemDataRole.DisplayRole, value)
            self.select_layout.layouts.addItem(item)
        for key, values in variants:
            value = values.split(":")[0]
            description = values.split(":")[1]
            if value in item.data(Qt.ItemDataRole.UserRole):
                # Workaround to prevent custom layout from using variants for English(US)
                if "custom" not in item.data(Qt.ItemDataRole.UserRole):
                    vitem = QListWidgetItem(description)
                    vitem.setData(Qt.ItemDataRole.UserRole, key)
                    vitem.setData(Qt.ItemDataRole.DisplayRole, description)
                    self.select_layout.variants.addItem(vitem)
        self.select_layout.layouts.setCurrentItem(self.select_layout.layouts.item(0))
        self.select_layout.variants.setCurrentItem(self.select_layout.variants.item(0))

        self.select_layout.layouts.currentItemChanged.connect(self.on_layout_changed)
        self.select_layout.buttonBox.rejected.connect(self.cancel)
        self.select_layout.buttonBox.accepted.connect(self.on_add_layout)

    def on_layout_changed(self):
        item = self.select_layout.layouts.currentItem()
        self.select_layout.variants.clear()
        none_item = QListWidgetItem()
        none_item.setData(Qt.ItemDataRole.UserRole, "")
        none_item.setData(Qt.ItemDataRole.DisplayRole, "")
        self.select_layout.variants.addItem(none_item)
        for key, values in variants:
            value = values.split(":")[0]
            description = values.split(":")[1]
            if value in item.data(Qt.ItemDataRole.UserRole):
                # Workaround to prevent custom layout from using variants for English(US)
                if "custom" not in item.data(Qt.ItemDataRole.UserRole):
                    vitem = QListWidgetItem(description)
                    vitem.setData(Qt.ItemDataRole.UserRole, key)
                    vitem.setData(Qt.ItemDataRole.DisplayRole, description)
                    self.select_layout.variants.addItem(vitem)
        self.select_layout.variants.setCurrentItem(self.select_layout.variants.item(0))

    def on_add_layout(self):
        self.accept()

    def cancel(self):
        self.reject()


def save_to_config():
    if settings["keyboard-use-settings"] == "true":

        lines = ['input "type:keyboard" {'] if not settings["keyboard-identifier"] else [
            'input "%s" {' % settings["keyboard-identifier"]]
        lines.append('  xkb_layout {}'.format(','.join(settings["keyboard-layout"])))
        if settings["keyboard-variant"] != []:
            lines.append('  xkb_variant {}'.format(','.join(settings["keyboard-variant"])))
        if settings["keyboard-shortcut"]:
            lines.append('  xkb_options {}'.format(settings["keyboard-shortcut"]))
        lines.append('  xkb_model {}'.format(settings["keyboard-model"]))
        lines.append('  repeat_delay {}'.format(settings["keyboard-repeat-delay"]))
        lines.append('  repeat_rate {}'.format(settings["keyboard-repeat-rate"]))
        lines.append('  xkb_capslock {}'.format(settings["keyboard-capslock"]))
        lines.append('  xkb_numlock {}'.format(settings["keyboard-numlock"]))
        lines.append('}')

        save_list_to_text_file(lines, os.path.join(config_home, "sway/keyboard"))

        config = load_text_file(sway_config).splitlines()
        new_config = []
        for line in config:
            if "include keyboard" not in line:
                new_config.append(line)
        new_config.append("include keyboard")
        save_list_to_text_file(new_config, sway_config)

    else:
        config = load_text_file(sway_config).splitlines()
        old_config = []
        for line in config:
            old_config.append(line)
        if "include keyboard" in old_config:
            old_config.remove("include keyboard")
        save_list_to_text_file(old_config, sway_config)

        if os.path.exists(os.path.join(config_home, "sway/keyboard")):
            os.unlink(os.path.join(config_home, "sway/keyboard"))

    if settings["pointer-use-settings"] == "true":

        lines = ['input "type:pointer" {'] if not settings["pointer-identifier"] else [
            'input "%s" {' % settings["pointer-identifier"]]
        lines.append('  accel_profile {}'.format(settings["pointer-accel-profile"])),
        lines.append('  pointer_accel {}'.format(settings["pointer-pointer-accel"])),
        lines.append('  natural_scroll {}'.format(settings["pointer-natural-scroll"])),
        lines.append('  scroll_factor {}'.format(settings["pointer-scroll-factor"])),
        lines.append('  scroll_button {}'.format(settings["pointer-scroll-button"])),
        lines.append('  left_handed {}'.format(settings["pointer-left-handed"])),
        lines.append('  middle_emulation {}'.format(settings["pointer-middle-emulation"])),
        if sway_version >= "1.9":
            lines.append('  rotation_angle {}'.format(settings["pointer-rotation-angle"]))
            lines.append('  scroll_button_lock {}'.format(settings["pointer-scroll-button-lock"])),
        lines.append('}')

        save_list_to_text_file(lines, os.path.join(config_home, "sway/pointer"))

        config = load_text_file(sway_config).splitlines()
        new_config = []
        for line in config:
            if "include pointer" not in line:
                new_config.append(line)
        new_config.append("include pointer")
        save_list_to_text_file(new_config, sway_config)

    else:
        config = load_text_file(sway_config).splitlines()
        old_config = []
        for line in config:
            old_config.append(line)
        if "include pointer" in old_config:
            old_config.remove("include pointer")
        save_list_to_text_file(old_config, sway_config)

        if os.path.exists(os.path.join(config_home, "sway/pointer")):
            os.unlink(os.path.join(config_home, "sway/pointer"))

    if settings["touchpad-use-settings"] == "true":
        lines = ['input "type:touchpad" {'] if not settings["touchpad-identifier"] else [
            'input "%s" {' % settings["touchpad-identifier"]]
        lines.append('  events {}'.format(settings["touchpad-events"])),
        lines.append('  accel_profile {}'.format(settings["touchpad-accel-profile"])),
        lines.append('  pointer_accel {}'.format(settings["touchpad-pointer-accel"])),
        lines.append('  natural_scroll {}'.format(settings["touchpad-natural-scroll"])),
        lines.append('  scroll_factor {}'.format(settings["touchpad-scroll-factor"])),
        lines.append('  scroll_method {}'.format(settings["touchpad-scroll-method"])),
        lines.append('  left_handed {}'.format(settings["touchpad-left-handed"])),
        lines.append('  tap {}'.format(settings["touchpad-tap"])),
        lines.append('  tap_button_map {}'.format(settings["touchpad-tap-button-map"])),
        lines.append('  drag {}'.format(settings["touchpad-drag"])),
        lines.append('  drag_lock {}'.format(settings["touchpad-drag-lock"])),
        lines.append('  dwt {}'.format(settings["touchpad-dwt"])),
        if sway_version >= "1.8":
            lines.append('  dwtp {}'.format(settings["touchpad-dwtp"])),
        if sway_version >= "1.9":
            lines.append('  rotation_angle {}'.format(settings["touchpad-rotation-angle"])),
        lines.append('  middle_emulation {}'.format(settings["touchpad-middle-emulation"])),
        lines.append('  click_method {}'.format(settings["touchpad-click-method"]))
        lines.append('}')

        save_list_to_text_file(lines, os.path.join(config_home, "sway/touchpad"))

        config = load_text_file(sway_config).splitlines()
        new_config = []
        for line in config:
            if "include touchpad" not in line:
                new_config.append(line)
        new_config.append("include touchpad")
        save_list_to_text_file(new_config, sway_config)

    else:
        config = load_text_file(sway_config).splitlines()
        old_config = []
        for line in config:
            old_config.append(line)
        if "include touchpad" in old_config:
            old_config.remove("include touchpad")
        save_list_to_text_file(old_config, sway_config)

        if os.path.exists(os.path.join(config_home, "sway/touchpad")):
            os.unlink(os.path.join(config_home, "sway/touchpad"))

    if settings["tablet-use-settings"] == "true":
        lines = ['input "type:tablet_tool" {'] if not settings["tablet-identifier"] else [
            'input "%s" {' % settings["tablet-identifier"]]
        lines.append('  left_handed {}'.format(settings["tablet-left-handed"]))
        lines.append('  tool_mode {}'.format(' '.join(settings["tablet-tool-mode"])))
        lines.append('}')

        save_list_to_text_file(lines, os.path.join(config_home, "sway/tablet"))

        config = load_text_file(sway_config).splitlines()
        new_config = []
        for line in config:
            if "include tablet" not in line:
                new_config.append(line)
        new_config.append("include tablet")
        save_list_to_text_file(new_config, sway_config)

    else:
        config = load_text_file(sway_config).splitlines()
        old_config = []
        for line in config:
            old_config.append(line)
        if "include tablet" in old_config:
            old_config.remove("include tablet")
        save_list_to_text_file(old_config, sway_config)

        if os.path.exists(os.path.join(config_home, "sway/tablet")):
            os.unlink(os.path.join(config_home, "sway/tablet"))


def load_settings():
    settings_file = os.path.join(data_dir, "settings")
    global settings
    if os.path.isfile(settings_file):
        print("Loading settings from {}".format(settings_file))
        settings = load_json(settings_file)
        defaults = load_json(default_settings)
        missing = 0
        for key in defaults:
            if key not in settings:
                settings[key] = defaults[key]
                print("'{}' key missing from settings, adding '{}'".format(key, defaults[key]))
                missing += 1
        if missing > 0:
            print("{} missing config key(s) substituted. Saving {}".format(missing, settings_file))
            save_json(settings, settings_file)
    else:
        print("Loading default settings")
        copy2(os.path.join(dir_name, "data/defaults.json"), os.path.join(data_dir, "settings"))
        settings = load_json(settings_file)


def main():
    app = QApplication(["Sway Input Configurator"])
    app.setDesktopFileName("sway-input-config")

    if os.getenv("SWAYSOCK"):
        parser = argparse.ArgumentParser()
        parser.add_argument("-v",
                            "--version",
                            action="version",
                            version=app_version,
                            help="display application version")
        parser.add_argument("-l",
                            "--locale",
                            default=QLocale.system().name(),
                            help="force application locale")
        parser.add_argument("-r",
                            "--restore",
                            action="store_true",
                            help="restore default settings")
        args = parser.parse_args()

        locale = args.locale
        locale_ts = QTranslator()
        app_ts = QTranslator()
        locale_ts.load('qt_%s' % locale, QLibraryInfo.path(QLibraryInfo.LibraryPath.TranslationsPath))
        app_ts.load('lang_%s' % locale, os.path.join(dir_name, "langs"))
        app.installTranslator(locale_ts)
        app.installTranslator(app_ts)

        global data_dir
        data_dir = get_data_dir()

        if args.restore:
            if input("\nRestore default settings? y/N ").upper() == "Y":
                copy2(os.path.join(dir_name, "data/defaults.json"), os.path.join(data_dir, "settings"))
                sys.exit(0)

        load_settings()
        win = MainWindow()
        win.show()
        sys.exit(app.exec())
    else:
        win = ErrorMessage()
        win.show()
        sys.exit(app.exec())


if __name__ == "__main__":
    signal.signal(signal.SIGINT, signal.SIG_DFL)
    main()



# Form implementation generated from reading ui file 'sway_input_config/ui/error_message.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_ErrorMessage(object):
    def setupUi(self, ErrorMessage):
        ErrorMessage.setObjectName("ErrorMessage")
        ErrorMessage.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
        ErrorMessage.resize(670, 115)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ErrorMessage.sizePolicy().hasHeightForWidth())
        ErrorMessage.setSizePolicy(sizePolicy)
        ErrorMessage.setMinimumSize(QtCore.QSize(670, 115))
        ErrorMessage.setMaximumSize(QtCore.QSize(670, 115))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("sway_input_config/ui/../data/logo_sic.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        ErrorMessage.setWindowIcon(icon)
        self.verticalLayout = QtWidgets.QVBoxLayout(ErrorMessage)
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(parent=ErrorMessage)
        self.label.setTextFormat(QtCore.Qt.TextFormat.AutoText)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=ErrorMessage)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.horizontalLayout_3.addWidget(self.buttonBox)
        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.retranslateUi(ErrorMessage)
        QtCore.QMetaObject.connectSlotsByName(ErrorMessage)

    def retranslateUi(self, ErrorMessage):
        _translate = QtCore.QCoreApplication.translate
        ErrorMessage.setWindowTitle(_translate("ErrorMessage", "Sway socket not found!"))
        self.label.setText(_translate("ErrorMessage", "<html><head/><body><p><span style=\" font-size:10pt;\">Sway Input Configurator only supports Sway. You are probably using an unsupported window manager or there are problems with your Sway configuration.</span></p></body></html>"))



# Form implementation generated from reading ui file 'sway_input_config/ui/mainwindow.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(910, 720)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(910, 720))
        MainWindow.setMaximumSize(QtCore.QSize(16777215, 16777215))
        MainWindow.setLocale(QtCore.QLocale(QtCore.QLocale.Language.English, QtCore.QLocale.Country.UnitedStates))
        self.centralwidget = QtWidgets.QWidget(parent=MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setMinimumSize(QtCore.QSize(615, 715))
        self.centralwidget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(parent=self.centralwidget)
        self.tabWidget.setStyleSheet("combobox-popup: 0")
        self.tabWidget.setObjectName("tabWidget")
        self.KeyboardTab = QtWidgets.QWidget()
        self.KeyboardTab.setObjectName("KeyboardTab")
        self.gridLayout = QtWidgets.QGridLayout(self.KeyboardTab)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.layouts = QtWidgets.QTreeWidget(parent=self.KeyboardTab)
        self.layouts.setDragEnabled(True)
        self.layouts.setDragDropMode(QtWidgets.QAbstractItemView.DragDropMode.InternalMove)
        self.layouts.setRootIsDecorated(False)
        self.layouts.setItemsExpandable(False)
        self.layouts.setColumnCount(2)
        self.layouts.setObjectName("layouts")
        self.horizontalLayout_2.addWidget(self.layouts)
        self.gridLayout.addLayout(self.horizontalLayout_2, 0, 0, 1, 1)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.addBtn = QtWidgets.QPushButton(parent=self.KeyboardTab)
        self.addBtn.setObjectName("addBtn")
        self.verticalLayout_5.addWidget(self.addBtn)
        self.rmBtn = QtWidgets.QPushButton(parent=self.KeyboardTab)
        self.rmBtn.setObjectName("rmBtn")
        self.verticalLayout_5.addWidget(self.rmBtn)
        self.upBtn = QtWidgets.QPushButton(parent=self.KeyboardTab)
        self.upBtn.setObjectName("upBtn")
        self.verticalLayout_5.addWidget(self.upBtn)
        self.downBtn = QtWidgets.QPushButton(parent=self.KeyboardTab)
        self.downBtn.setObjectName("downBtn")
        self.verticalLayout_5.addWidget(self.downBtn)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_5.addItem(spacerItem)
        self.gridLayout.addLayout(self.verticalLayout_5, 0, 1, 1, 1)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.caps_lock = QtWidgets.QCheckBox(parent=self.KeyboardTab)
        self.caps_lock.setText("")
        self.caps_lock.setObjectName("caps_lock")
        self.gridLayout_2.addWidget(self.caps_lock, 5, 1, 1, 1)
        self.shortcutLabel = QtWidgets.QLabel(parent=self.KeyboardTab)
        self.shortcutLabel.setObjectName("shortcutLabel")
        self.gridLayout_2.addWidget(self.shortcutLabel, 2, 0, 1, 1)
        self.repeatDelay = QtWidgets.QSpinBox(parent=self.KeyboardTab)
        self.repeatDelay.setMinimum(1)
        self.repeatDelay.setMaximum(6000)
        self.repeatDelay.setObjectName("repeatDelay")
        self.gridLayout_2.addWidget(self.repeatDelay, 3, 2, 1, 1)
        self.caps_lockLabel = QtWidgets.QLabel(parent=self.KeyboardTab)
        self.caps_lockLabel.setObjectName("caps_lockLabel")
        self.gridLayout_2.addWidget(self.caps_lockLabel, 5, 0, 1, 1)
        self.repeatRateSlider = QtWidgets.QSlider(parent=self.KeyboardTab)
        self.repeatRateSlider.setMinimum(1)
        self.repeatRateSlider.setMaximum(4000)
        self.repeatRateSlider.setPageStep(50)
        self.repeatRateSlider.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.repeatRateSlider.setObjectName("repeatRateSlider")
        self.gridLayout_2.addWidget(self.repeatRateSlider, 4, 1, 1, 1)
        self.repeatRate = QtWidgets.QSpinBox(parent=self.KeyboardTab)
        self.repeatRate.setMinimum(1)
        self.repeatRate.setMaximum(4000)
        self.repeatRate.setObjectName("repeatRate")
        self.gridLayout_2.addWidget(self.repeatRate, 4, 2, 1, 1)
        self.kbdID = QtWidgets.QComboBox(parent=self.KeyboardTab)
        self.kbdID.setStyleSheet("combobox-popup: 0")
        self.kbdID.setObjectName("kbdID")
        self.gridLayout_2.addWidget(self.kbdID, 0, 1, 1, 1)
        self.repeatDelaySlider = QtWidgets.QSlider(parent=self.KeyboardTab)
        self.repeatDelaySlider.setMinimum(1)
        self.repeatDelaySlider.setMaximum(6000)
        self.repeatDelaySlider.setPageStep(50)
        self.repeatDelaySlider.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.repeatDelaySlider.setObjectName("repeatDelaySlider")
        self.gridLayout_2.addWidget(self.repeatDelaySlider, 3, 1, 1, 1)
        self.repeatDelayLabel = QtWidgets.QLabel(parent=self.KeyboardTab)
        self.repeatDelayLabel.setObjectName("repeatDelayLabel")
        self.gridLayout_2.addWidget(self.repeatDelayLabel, 3, 0, 1, 1)
        self.kbdModel = QtWidgets.QComboBox(parent=self.KeyboardTab)
        self.kbdModel.setStyleSheet("combobox-popup: 0;")
        self.kbdModel.setObjectName("kbdModel")
        self.gridLayout_2.addWidget(self.kbdModel, 1, 1, 1, 1)
        self.kbdID_label = QtWidgets.QLabel(parent=self.KeyboardTab)
        self.kbdID_label.setObjectName("kbdID_label")
        self.gridLayout_2.addWidget(self.kbdID_label, 0, 0, 1, 1)
        self.repeatRateLabel = QtWidgets.QLabel(parent=self.KeyboardTab)
        self.repeatRateLabel.setObjectName("repeatRateLabel")
        self.gridLayout_2.addWidget(self.repeatRateLabel, 4, 0, 1, 1)
        self.num_lockLabel = QtWidgets.QLabel(parent=self.KeyboardTab)
        self.num_lockLabel.setObjectName("num_lockLabel")
        self.gridLayout_2.addWidget(self.num_lockLabel, 6, 0, 1, 1)
        self.num_lock = QtWidgets.QCheckBox(parent=self.KeyboardTab)
        self.num_lock.setText("")
        self.num_lock.setObjectName("num_lock")
        self.gridLayout_2.addWidget(self.num_lock, 6, 1, 1, 1)
        self.shortcutName = QtWidgets.QComboBox(parent=self.KeyboardTab)
        self.shortcutName.setStyleSheet("combobox-popup: 0;")
        self.shortcutName.setObjectName("shortcutName")
        self.gridLayout_2.addWidget(self.shortcutName, 2, 1, 1, 1)
        self.kbdModel_label = QtWidgets.QLabel(parent=self.KeyboardTab)
        self.kbdModel_label.setObjectName("kbdModel_label")
        self.gridLayout_2.addWidget(self.kbdModel_label, 1, 0, 1, 1)
        self.gridLayout.addLayout(self.gridLayout_2, 1, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.KeyBoardUseSettings = QtWidgets.QCheckBox(parent=self.KeyboardTab)
        self.KeyBoardUseSettings.setLayoutDirection(QtCore.Qt.LayoutDirection.LeftToRight)
        self.KeyBoardUseSettings.setObjectName("KeyBoardUseSettings")
        self.horizontalLayout_3.addWidget(self.KeyBoardUseSettings)
        self.gridLayout.addLayout(self.horizontalLayout_3, 2, 0, 1, 2)
        self.tabWidget.addTab(self.KeyboardTab, "")
        self.PointerTab = QtWidgets.QWidget()
        self.PointerTab.setObjectName("PointerTab")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.PointerTab)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.groupBox_5 = QtWidgets.QGroupBox(parent=self.PointerTab)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.pointerID = QtWidgets.QComboBox(parent=self.groupBox_5)
        self.pointerID.setStyleSheet("combobox-popup: 0")
        self.pointerID.setObjectName("pointerID")
        self.gridLayout_3.addWidget(self.pointerID, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label.setObjectName("label")
        self.gridLayout_3.addWidget(self.label, 1, 0, 1, 1)
        self.pointerID_label = QtWidgets.QLabel(parent=self.groupBox_5)
        self.pointerID_label.setToolTip("")
        self.pointerID_label.setObjectName("pointerID_label")
        self.gridLayout_3.addWidget(self.pointerID_label, 0, 0, 1, 1)
        self.pointerLeftHanded = QtWidgets.QCheckBox(parent=self.groupBox_5)
        self.pointerLeftHanded.setObjectName("pointerLeftHanded")
        self.gridLayout_3.addWidget(self.pointerLeftHanded, 1, 1, 1, 1)
        self.pointerMiddle = QtWidgets.QCheckBox(parent=self.groupBox_5)
        self.pointerMiddle.setObjectName("pointerMiddle")
        self.gridLayout_3.addWidget(self.pointerMiddle, 2, 1, 1, 1)
        self.gridLayout_11.addWidget(self.groupBox_5, 0, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.PointerUseSettings = QtWidgets.QCheckBox(parent=self.PointerTab)
        self.PointerUseSettings.setObjectName("PointerUseSettings")
        self.horizontalLayout_4.addWidget(self.PointerUseSettings)
        self.gridLayout_11.addLayout(self.horizontalLayout_4, 3, 0, 1, 2)
        self.groupBox_7 = QtWidgets.QGroupBox(parent=self.PointerTab)
        self.groupBox_7.setObjectName("groupBox_7")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_7)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.label_4 = QtWidgets.QLabel(parent=self.groupBox_7)
        self.label_4.setObjectName("label_4")
        self.gridLayout_10.addWidget(self.label_4, 0, 0, 1, 1)
        self.pointerNatScroll = QtWidgets.QCheckBox(parent=self.groupBox_7)
        self.pointerNatScroll.setObjectName("pointerNatScroll")
        self.gridLayout_10.addWidget(self.pointerNatScroll, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(parent=self.groupBox_7)
        self.label_5.setObjectName("label_5")
        self.gridLayout_10.addWidget(self.label_5, 1, 0, 1, 1)
        self.pointerScrollFactor = QtWidgets.QSlider(parent=self.groupBox_7)
        self.pointerScrollFactor.setMinimum(1)
        self.pointerScrollFactor.setMaximum(100)
        self.pointerScrollFactor.setPageStep(1)
        self.pointerScrollFactor.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.pointerScrollFactor.setTickPosition(QtWidgets.QSlider.TickPosition.TicksBelow)
        self.pointerScrollFactor.setTickInterval(9)
        self.pointerScrollFactor.setObjectName("pointerScrollFactor")
        self.gridLayout_10.addWidget(self.pointerScrollFactor, 1, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(parent=self.groupBox_7)
        self.label_9.setObjectName("label_9")
        self.gridLayout_10.addWidget(self.label_9, 2, 0, 1, 1)
        self.scrollButtonList = QtWidgets.QComboBox(parent=self.groupBox_7)
        self.scrollButtonList.setEnabled(True)
        self.scrollButtonList.setObjectName("scrollButtonList")
        self.gridLayout_10.addWidget(self.scrollButtonList, 2, 1, 1, 1)
        self.scrollButtonLock = QtWidgets.QCheckBox(parent=self.groupBox_7)
        self.scrollButtonLock.setEnabled(False)
        self.scrollButtonLock.setObjectName("scrollButtonLock")
        self.gridLayout_10.addWidget(self.scrollButtonLock, 3, 1, 1, 1)
        self.gridLayout_11.addWidget(self.groupBox_7, 0, 1, 1, 1)
        self.groupBox_6 = QtWidgets.QGroupBox(parent=self.PointerTab)
        self.groupBox_6.setObjectName("groupBox_6")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_6)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_2 = QtWidgets.QLabel(parent=self.groupBox_6)
        self.label_2.setObjectName("label_2")
        self.gridLayout_4.addWidget(self.label_2, 0, 0, 1, 1)
        self.pointerAccel = QtWidgets.QSlider(parent=self.groupBox_6)
        self.pointerAccel.setMinimum(-10)
        self.pointerAccel.setMaximum(10)
        self.pointerAccel.setPageStep(1)
        self.pointerAccel.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.pointerAccel.setTickPosition(QtWidgets.QSlider.TickPosition.TicksBelow)
        self.pointerAccel.setTickInterval(1)
        self.pointerAccel.setObjectName("pointerAccel")
        self.gridLayout_4.addWidget(self.pointerAccel, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.groupBox_6)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 1, 0, 1, 1)
        self.pointerFlat = QtWidgets.QRadioButton(parent=self.groupBox_6)
        self.pointerFlat.setObjectName("pointerFlat")
        self.gridLayout_4.addWidget(self.pointerFlat, 1, 1, 1, 1)
        self.pointerAdaptive = QtWidgets.QRadioButton(parent=self.groupBox_6)
        self.pointerAdaptive.setObjectName("pointerAdaptive")
        self.gridLayout_4.addWidget(self.pointerAdaptive, 2, 1, 1, 1)
        self.pointerRotationLabel = QtWidgets.QLabel(parent=self.groupBox_6)
        self.pointerRotationLabel.setObjectName("pointerRotationLabel")
        self.gridLayout_4.addWidget(self.pointerRotationLabel, 3, 0, 1, 1)
        self.pointerRotationAngleSlider = QtWidgets.QSlider(parent=self.groupBox_6)
        self.pointerRotationAngleSlider.setMaximum(360)
        self.pointerRotationAngleSlider.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.pointerRotationAngleSlider.setTickPosition(QtWidgets.QSlider.TickPosition.TicksBelow)
        self.pointerRotationAngleSlider.setTickInterval(10)
        self.pointerRotationAngleSlider.setObjectName("pointerRotationAngleSlider")
        self.gridLayout_4.addWidget(self.pointerRotationAngleSlider, 3, 1, 1, 1)
        self.pointerRotationAngle = QtWidgets.QSpinBox(parent=self.groupBox_6)
        self.pointerRotationAngle.setMaximum(360)
        self.pointerRotationAngle.setObjectName("pointerRotationAngle")
        self.gridLayout_4.addWidget(self.pointerRotationAngle, 3, 2, 1, 1)
        self.gridLayout_11.addWidget(self.groupBox_6, 1, 0, 1, 2)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_11.addItem(spacerItem3, 2, 0, 1, 2)
        self.tabWidget.addTab(self.PointerTab, "")
        self.TouchpadTab = QtWidgets.QWidget()
        self.TouchpadTab.setObjectName("TouchpadTab")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.TouchpadTab)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem4 = QtWidgets.QSpacerItem(271, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.TouchPadUseSettings = QtWidgets.QCheckBox(parent=self.TouchpadTab)
        self.TouchPadUseSettings.setObjectName("TouchPadUseSettings")
        self.horizontalLayout.addWidget(self.TouchPadUseSettings)
        self.gridLayout_9.addLayout(self.horizontalLayout, 4, 0, 1, 2)
        self.touchButtonBox = QtWidgets.QGroupBox(parent=self.TouchpadTab)
        self.touchButtonBox.setObjectName("touchButtonBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.touchButtonBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.lrm = QtWidgets.QRadioButton(parent=self.touchButtonBox)
        self.lrm.setAutoExclusive(True)
        self.lrm.setObjectName("lrm")
        self.gridLayout_7.addWidget(self.lrm, 4, 1, 1, 1)
        self.touchLeftHanded = QtWidgets.QCheckBox(parent=self.touchButtonBox)
        self.touchLeftHanded.setObjectName("touchLeftHanded")
        self.gridLayout_7.addWidget(self.touchLeftHanded, 2, 1, 1, 1)
        self.tap_click = QtWidgets.QCheckBox(parent=self.touchButtonBox)
        self.tap_click.setObjectName("tap_click")
        self.gridLayout_7.addWidget(self.tap_click, 0, 0, 1, 1)
        self.label_13 = QtWidgets.QLabel(parent=self.touchButtonBox)
        self.label_13.setObjectName("label_13")
        self.gridLayout_7.addWidget(self.label_13, 6, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(parent=self.touchButtonBox)
        self.label_10.setObjectName("label_10")
        self.gridLayout_7.addWidget(self.label_10, 4, 0, 1, 1)
        self.btn_ClickFinger = QtWidgets.QRadioButton(parent=self.touchButtonBox)
        self.btn_ClickFinger.setObjectName("btn_ClickFinger")
        self.gridLayout_7.addWidget(self.btn_ClickFinger, 7, 1, 1, 1)
        self.DWT = QtWidgets.QCheckBox(parent=self.touchButtonBox)
        self.DWT.setObjectName("DWT")
        self.gridLayout_7.addWidget(self.DWT, 0, 1, 1, 1)
        self.drag_lock = QtWidgets.QCheckBox(parent=self.touchButtonBox)
        self.drag_lock.setObjectName("drag_lock")
        self.gridLayout_7.addWidget(self.drag_lock, 2, 0, 1, 1)
        self.DWTP = QtWidgets.QCheckBox(parent=self.touchButtonBox)
        self.DWTP.setObjectName("DWTP")
        self.gridLayout_7.addWidget(self.DWTP, 1, 1, 1, 1)
        self.drag = QtWidgets.QCheckBox(parent=self.touchButtonBox)
        self.drag.setObjectName("drag")
        self.gridLayout_7.addWidget(self.drag, 1, 0, 1, 1)
        self.touchMiddle = QtWidgets.QCheckBox(parent=self.touchButtonBox)
        self.touchMiddle.setLayoutDirection(QtCore.Qt.LayoutDirection.LeftToRight)
        self.touchMiddle.setObjectName("touchMiddle")
        self.gridLayout_7.addWidget(self.touchMiddle, 3, 1, 1, 1)
        self.btn_BtnArea = QtWidgets.QRadioButton(parent=self.touchButtonBox)
        self.btn_BtnArea.setObjectName("btn_BtnArea")
        self.gridLayout_7.addWidget(self.btn_BtnArea, 6, 1, 1, 1)
        self.lmr = QtWidgets.QRadioButton(parent=self.touchButtonBox)
        self.lmr.setAutoExclusive(True)
        self.lmr.setObjectName("lmr")
        self.gridLayout_7.addWidget(self.lmr, 5, 1, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_7.addItem(spacerItem5, 8, 0, 1, 2)
        self.gridLayout_9.addWidget(self.touchButtonBox, 0, 1, 2, 1)
        self.touchGeneralBox = QtWidgets.QGroupBox(parent=self.TouchpadTab)
        self.touchGeneralBox.setObjectName("touchGeneralBox")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.touchGeneralBox)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.touchpadID_label = QtWidgets.QLabel(parent=self.touchGeneralBox)
        self.touchpadID_label.setObjectName("touchpadID_label")
        self.gridLayout_6.addWidget(self.touchpadID_label, 0, 0, 1, 1)
        self.touchpadID = QtWidgets.QComboBox(parent=self.touchGeneralBox)
        self.touchpadID.setObjectName("touchpadID")
        self.gridLayout_6.addWidget(self.touchpadID, 0, 1, 1, 1)
        self.eventLabel = QtWidgets.QLabel(parent=self.touchGeneralBox)
        self.eventLabel.setObjectName("eventLabel")
        self.gridLayout_6.addWidget(self.eventLabel, 1, 0, 1, 1)
        self.touchEventsEnabled = QtWidgets.QRadioButton(parent=self.touchGeneralBox)
        self.touchEventsEnabled.setAutoExclusive(True)
        self.touchEventsEnabled.setObjectName("touchEventsEnabled")
        self.gridLayout_6.addWidget(self.touchEventsEnabled, 1, 1, 1, 1)
        self.touchEventsDisabled = QtWidgets.QRadioButton(parent=self.touchGeneralBox)
        self.touchEventsDisabled.setAutoExclusive(True)
        self.touchEventsDisabled.setObjectName("touchEventsDisabled")
        self.gridLayout_6.addWidget(self.touchEventsDisabled, 2, 1, 1, 1)
        self.touchEventsOnExternalMouse = QtWidgets.QRadioButton(parent=self.touchGeneralBox)
        self.touchEventsOnExternalMouse.setAutoExclusive(True)
        self.touchEventsOnExternalMouse.setObjectName("touchEventsOnExternalMouse")
        self.gridLayout_6.addWidget(self.touchEventsOnExternalMouse, 3, 1, 1, 1)
        self.gridLayout_9.addWidget(self.touchGeneralBox, 0, 0, 1, 1)
        self.touchPointerBox = QtWidgets.QGroupBox(parent=self.TouchpadTab)
        self.touchPointerBox.setObjectName("touchPointerBox")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.touchPointerBox)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_7 = QtWidgets.QLabel(parent=self.touchPointerBox)
        self.label_7.setObjectName("label_7")
        self.gridLayout_5.addWidget(self.label_7, 0, 0, 1, 1)
        self.touchAccel = QtWidgets.QSlider(parent=self.touchPointerBox)
        self.touchAccel.setMinimum(-10)
        self.touchAccel.setMaximum(10)
        self.touchAccel.setPageStep(1)
        self.touchAccel.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.touchAccel.setTickPosition(QtWidgets.QSlider.TickPosition.TicksBelow)
        self.touchAccel.setTickInterval(1)
        self.touchAccel.setObjectName("touchAccel")
        self.gridLayout_5.addWidget(self.touchAccel, 0, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(parent=self.touchPointerBox)
        self.label_8.setObjectName("label_8")
        self.gridLayout_5.addWidget(self.label_8, 1, 0, 1, 1)
        self.touchFlat = QtWidgets.QRadioButton(parent=self.touchPointerBox)
        self.touchFlat.setAutoExclusive(True)
        self.touchFlat.setObjectName("touchFlat")
        self.gridLayout_5.addWidget(self.touchFlat, 1, 1, 1, 1)
        self.touchAdaptive = QtWidgets.QRadioButton(parent=self.touchPointerBox)
        self.touchAdaptive.setAutoExclusive(True)
        self.touchAdaptive.setObjectName("touchAdaptive")
        self.gridLayout_5.addWidget(self.touchAdaptive, 2, 1, 1, 1)
        self.touchpadRotationLabel = QtWidgets.QLabel(parent=self.touchPointerBox)
        self.touchpadRotationLabel.setObjectName("touchpadRotationLabel")
        self.gridLayout_5.addWidget(self.touchpadRotationLabel, 3, 0, 1, 1)
        self.touchRotationAngleSlider = QtWidgets.QSlider(parent=self.touchPointerBox)
        self.touchRotationAngleSlider.setMaximum(360)
        self.touchRotationAngleSlider.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.touchRotationAngleSlider.setTickPosition(QtWidgets.QSlider.TickPosition.TicksBelow)
        self.touchRotationAngleSlider.setTickInterval(10)
        self.touchRotationAngleSlider.setObjectName("touchRotationAngleSlider")
        self.gridLayout_5.addWidget(self.touchRotationAngleSlider, 3, 1, 1, 1)
        self.touchRotationAngle = QtWidgets.QSpinBox(parent=self.touchPointerBox)
        self.touchRotationAngle.setMaximum(360)
        self.touchRotationAngle.setObjectName("touchRotationAngle")
        self.gridLayout_5.addWidget(self.touchRotationAngle, 3, 2, 1, 1)
        self.gridLayout_9.addWidget(self.touchPointerBox, 1, 0, 1, 1)
        self.touchScrollingBox = QtWidgets.QGroupBox(parent=self.TouchpadTab)
        self.touchScrollingBox.setObjectName("touchScrollingBox")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.touchScrollingBox)
        self.gridLayout_8.setObjectName("gridLayout_8")
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_8.addItem(spacerItem6, 0, 4, 1, 1)
        self.touchScrollFactor = QtWidgets.QSlider(parent=self.touchScrollingBox)
        self.touchScrollFactor.setMinimum(1)
        self.touchScrollFactor.setMaximum(100)
        self.touchScrollFactor.setPageStep(1)
        self.touchScrollFactor.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.touchScrollFactor.setTickPosition(QtWidgets.QSlider.TickPosition.TicksBelow)
        self.touchScrollFactor.setTickInterval(9)
        self.touchScrollFactor.setObjectName("touchScrollFactor")
        self.gridLayout_8.addWidget(self.touchScrollFactor, 0, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(parent=self.touchScrollingBox)
        self.label_6.setObjectName("label_6")
        self.gridLayout_8.addWidget(self.label_6, 1, 1, 1, 1)
        self.touchNatScroll = QtWidgets.QCheckBox(parent=self.touchScrollingBox)
        self.touchNatScroll.setObjectName("touchNatScroll")
        self.gridLayout_8.addWidget(self.touchNatScroll, 1, 2, 1, 2)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_8.addItem(spacerItem7, 0, 0, 1, 1)
        self.label_12 = QtWidgets.QLabel(parent=self.touchScrollingBox)
        self.label_12.setObjectName("label_12")
        self.gridLayout_8.addWidget(self.label_12, 0, 1, 1, 1)
        self.label_11 = QtWidgets.QLabel(parent=self.touchScrollingBox)
        self.label_11.setObjectName("label_11")
        self.gridLayout_8.addWidget(self.label_11, 2, 1, 1, 1)
        self.touchScrollNoScroll = QtWidgets.QRadioButton(parent=self.touchScrollingBox)
        self.touchScrollNoScroll.setAutoExclusive(True)
        self.touchScrollNoScroll.setObjectName("touchScrollNoScroll")
        self.gridLayout_8.addWidget(self.touchScrollNoScroll, 2, 2, 1, 1)
        self.touchScrollTwoFingers = QtWidgets.QRadioButton(parent=self.touchScrollingBox)
        self.touchScrollTwoFingers.setAutoExclusive(True)
        self.touchScrollTwoFingers.setObjectName("touchScrollTwoFingers")
        self.gridLayout_8.addWidget(self.touchScrollTwoFingers, 3, 2, 1, 1)
        self.touchScrollEdges = QtWidgets.QRadioButton(parent=self.touchScrollingBox)
        self.touchScrollEdges.setAutoExclusive(True)
        self.touchScrollEdges.setObjectName("touchScrollEdges")
        self.gridLayout_8.addWidget(self.touchScrollEdges, 4, 2, 1, 1)
        self.gridLayout_9.addWidget(self.touchScrollingBox, 2, 0, 1, 2)
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_9.addItem(spacerItem8, 3, 0, 1, 2)
        self.tabWidget.addTab(self.TouchpadTab, "")
        self.TabletTab = QtWidgets.QWidget()
        self.TabletTab.setObjectName("TabletTab")
        self.gridLayout_13 = QtWidgets.QGridLayout(self.TabletTab)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem9)
        self.TabletUseSettings = QtWidgets.QCheckBox(parent=self.TabletTab)
        self.TabletUseSettings.setObjectName("TabletUseSettings")
        self.horizontalLayout_5.addWidget(self.TabletUseSettings)
        self.gridLayout_13.addLayout(self.horizontalLayout_5, 2, 0, 1, 3)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_13.addItem(spacerItem10, 1, 0, 1, 3)
        self.tabletGeneralBox = QtWidgets.QGroupBox(parent=self.TabletTab)
        self.tabletGeneralBox.setObjectName("tabletGeneralBox")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.tabletGeneralBox)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.label_14 = QtWidgets.QLabel(parent=self.tabletGeneralBox)
        self.label_14.setObjectName("label_14")
        self.gridLayout_12.addWidget(self.label_14, 0, 0, 1, 1)
        self.toolMoveRelative = QtWidgets.QRadioButton(parent=self.tabletGeneralBox)
        self.toolMoveRelative.setObjectName("toolMoveRelative")
        self.gridLayout_12.addWidget(self.toolMoveRelative, 3, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(parent=self.tabletGeneralBox)
        self.label_15.setObjectName("label_15")
        self.gridLayout_12.addWidget(self.label_15, 1, 0, 1, 1)
        self.toolModeList = QtWidgets.QComboBox(parent=self.tabletGeneralBox)
        self.toolModeList.setObjectName("toolModeList")
        self.gridLayout_12.addWidget(self.toolModeList, 1, 1, 1, 1)
        self.tabletID = QtWidgets.QComboBox(parent=self.tabletGeneralBox)
        self.tabletID.setObjectName("tabletID")
        self.gridLayout_12.addWidget(self.tabletID, 0, 1, 1, 1)
        self.tabletLeftHanded = QtWidgets.QCheckBox(parent=self.tabletGeneralBox)
        self.tabletLeftHanded.setObjectName("tabletLeftHanded")
        self.gridLayout_12.addWidget(self.tabletLeftHanded, 4, 1, 1, 1)
        self.label_16 = QtWidgets.QLabel(parent=self.tabletGeneralBox)
        self.label_16.setObjectName("label_16")
        self.gridLayout_12.addWidget(self.label_16, 2, 0, 1, 1)
        self.toolMoveAbsolute = QtWidgets.QRadioButton(parent=self.tabletGeneralBox)
        self.toolMoveAbsolute.setObjectName("toolMoveAbsolute")
        self.gridLayout_12.addWidget(self.toolMoveAbsolute, 2, 1, 1, 1)
        self.gridLayout_13.addWidget(self.tabletGeneralBox, 0, 0, 1, 3)
        self.tabWidget.addTab(self.TabletTab, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=self.centralwidget)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Apply|QtWidgets.QDialogButtonBox.StandardButton.Close|QtWidgets.QDialogButtonBox.StandardButton.Help|QtWidgets.QDialogButtonBox.StandardButton.RestoreDefaults)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Sway Input Configurator"))
        self.layouts.headerItem().setText(0, _translate("MainWindow", "Layout"))
        self.layouts.headerItem().setText(1, _translate("MainWindow", "Variant"))
        self.addBtn.setToolTip(_translate("MainWindow", "Add layout to the list"))
        self.addBtn.setText(_translate("MainWindow", "Add"))
        self.rmBtn.setToolTip(_translate("MainWindow", "Remove layout from the list."))
        self.rmBtn.setText(_translate("MainWindow", "Remove"))
        self.upBtn.setToolTip(_translate("MainWindow", "Move selected layout up."))
        self.upBtn.setText(_translate("MainWindow", "Up"))
        self.downBtn.setToolTip(_translate("MainWindow", "Move selected layout down."))
        self.downBtn.setText(_translate("MainWindow", "Down"))
        self.caps_lock.setToolTip(_translate("MainWindow", "Initially enables or disables CapsLock on startup."))
        self.shortcutLabel.setText(_translate("MainWindow", "Switching layouts:"))
        self.repeatDelay.setToolTip(_translate("MainWindow", "Amount of time a key must be held before it starts repeating."))
        self.repeatDelay.setSuffix(_translate("MainWindow", " ms"))
        self.caps_lockLabel.setText(_translate("MainWindow", "Enable CapsLock on system startup:"))
        self.repeatRate.setToolTip(_translate("MainWindow", "Frequency of key repeats once the repeat_delay has passed."))
        self.repeatRate.setSuffix(_translate("MainWindow", " repeats/s"))
        self.kbdID.setToolTip(_translate("MainWindow", "Keyboard identifier"))
        self.repeatDelayLabel.setText(_translate("MainWindow", "Repeat delay:"))
        self.kbdID_label.setText(_translate("MainWindow", "Keyboard ID:"))
        self.repeatRateLabel.setText(_translate("MainWindow", "Repeat rate:"))
        self.num_lockLabel.setText(_translate("MainWindow", "Enable NumLock on system startup:"))
        self.num_lock.setToolTip(_translate("MainWindow", "Initially enables or disables NumLock on startup."))
        self.shortcutName.setToolTip(_translate("MainWindow", "Keyboard shortcut to switch between layouts."))
        self.kbdModel_label.setText(_translate("MainWindow", "Keyboard model:"))
        self.KeyBoardUseSettings.setText(_translate("MainWindow", "Use these settings"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.KeyboardTab), _translate("MainWindow", "Keyboard"))
        self.groupBox_5.setTitle(_translate("MainWindow", "General"))
        self.pointerID.setToolTip(_translate("MainWindow", "Pointer device identifier"))
        self.label.setText(_translate("MainWindow", "Buttons:"))
        self.pointerID_label.setText(_translate("MainWindow", "Pointer device ID:"))
        self.pointerLeftHanded.setToolTip(_translate("MainWindow", "Enables or disables left handed mode."))
        self.pointerLeftHanded.setText(_translate("MainWindow", "Left handed mode"))
        self.pointerMiddle.setToolTip(_translate("MainWindow", "Enables or disables middle click emulation."))
        self.pointerMiddle.setText(_translate("MainWindow", "Press left and right buttons for middle click"))
        self.PointerUseSettings.setText(_translate("MainWindow", "Use these settings"))
        self.groupBox_7.setTitle(_translate("MainWindow", "Scrolling"))
        self.label_4.setText(_translate("MainWindow", "Scrolling:"))
        self.pointerNatScroll.setToolTip(_translate("MainWindow", "Touchscreen like scrolling."))
        self.pointerNatScroll.setText(_translate("MainWindow", "Invert scroll direction"))
        self.label_5.setText(_translate("MainWindow", "Scrolling speed:"))
        self.pointerScrollFactor.setToolTip(_translate("MainWindow", "Scroll speed will be scaled by the given value."))
        self.label_9.setText(_translate("MainWindow", "Scroll button (trackpoints only):"))
        self.scrollButtonList.setToolTip(_translate("MainWindow", "Sets the button used for scroll_method on_button_down."))
        self.scrollButtonLock.setToolTip(_translate("MainWindow", "Enables or disables scroll button lock for specified input device."))
        self.scrollButtonLock.setText(_translate("MainWindow", "Scroll button lock"))
        self.groupBox_6.setTitle(_translate("MainWindow", "Pointer"))
        self.label_2.setText(_translate("MainWindow", "Pointer speed:"))
        self.pointerAccel.setToolTip(_translate("MainWindow", "Changes the pointer acceleration."))
        self.label_3.setText(_translate("MainWindow", "Acceleration profile:"))
        self.pointerFlat.setToolTip(_translate("MainWindow", "Cursor moves the same distance as the mouse movement."))
        self.pointerFlat.setText(_translate("MainWindow", "Flat"))
        self.pointerAdaptive.setToolTip(_translate("MainWindow", "Cursor travel distance depends on the mouse movement speed."))
        self.pointerAdaptive.setText(_translate("MainWindow", "Adaptive"))
        self.pointerRotationLabel.setText(_translate("MainWindow", "Rotation angle:"))
        self.pointerRotationAngleSlider.setToolTip(_translate("MainWindow", "Sets the rotation angle of the device to the given angle, in degrees clockwise."))
        self.pointerRotationAngle.setToolTip(_translate("MainWindow", "Sets the rotation angle of the device to the given angle, in degrees clockwise."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.PointerTab), _translate("MainWindow", "Pointer device"))
        self.TouchPadUseSettings.setText(_translate("MainWindow", "Use these settings"))
        self.touchButtonBox.setTitle(_translate("MainWindow", "Touchpad buttons"))
        self.lrm.setText(_translate("MainWindow", "2 fingers - right click, 3 - middle"))
        self.touchLeftHanded.setToolTip(_translate("MainWindow", "Enables or disables left handed mode."))
        self.touchLeftHanded.setText(_translate("MainWindow", "Left handed mode"))
        self.tap_click.setText(_translate("MainWindow", "Tap-to-click"))
        self.label_13.setText(_translate("MainWindow", "Click method\n"
"(clickpads only):"))
        self.label_10.setText(_translate("MainWindow", "Tap button mapping:"))
        self.btn_ClickFinger.setToolTip(_translate("MainWindow", "Clicking with 1, 2, 3 fingers triggers a left, right, or middle click, respectively"))
        self.btn_ClickFinger.setText(_translate("MainWindow", "Click fingers"))
        self.DWT.setToolTip(_translate("MainWindow", "Disables touchpad while typing the text"))
        self.DWT.setText(_translate("MainWindow", "Disable while typing"))
        self.drag_lock.setText(_translate("MainWindow", "Tap-and-drag lock"))
        self.DWTP.setToolTip(_translate("MainWindow", "Disables touchpad while using trackpoint device"))
        self.DWTP.setText(_translate("MainWindow", "Disable while trackpointing"))
        self.drag.setText(_translate("MainWindow", "Tap-and-drag"))
        self.touchMiddle.setToolTip(_translate("MainWindow", "Enables or disables middle click emulation."))
        self.touchMiddle.setText(_translate("MainWindow", " Press left and right buttons for\n"
" middle click"))
        self.btn_BtnArea.setToolTip(_translate("MainWindow", "The bottom area of the touchpad is divided into a left, middle and right button area"))
        self.btn_BtnArea.setText(_translate("MainWindow", "Button area"))
        self.lmr.setText(_translate("MainWindow", "2 fingers - middle click, 3 - right"))
        self.touchGeneralBox.setTitle(_translate("MainWindow", "General"))
        self.touchpadID_label.setText(_translate("MainWindow", "Touchpad ID:"))
        self.touchpadID.setToolTip(_translate("MainWindow", "Touchpad identifier"))
        self.eventLabel.setText(_translate("MainWindow", "Touchpad events:"))
        self.touchEventsEnabled.setToolTip(_translate("MainWindow", "Send events normally"))
        self.touchEventsEnabled.setText(_translate("MainWindow", "Enable"))
        self.touchEventsDisabled.setToolTip(_translate("MainWindow", "Touchpad only stops sending events but not get fully disabled"))
        self.touchEventsDisabled.setText(_translate("MainWindow", "Disable"))
        self.touchEventsOnExternalMouse.setToolTip(_translate("MainWindow", "Disable touchpad while an external mouse is plugged in"))
        self.touchEventsOnExternalMouse.setText(_translate("MainWindow", "Disable when external mouse is plugged in"))
        self.touchPointerBox.setTitle(_translate("MainWindow", "Pointer"))
        self.label_7.setText(_translate("MainWindow", "Pointer speed:"))
        self.touchAccel.setToolTip(_translate("MainWindow", "Changes the pointer acceleration."))
        self.label_8.setText(_translate("MainWindow", "Acceleration profile:"))
        self.touchFlat.setToolTip(_translate("MainWindow", "Cursor moves the same distance as the mouse movement."))
        self.touchFlat.setText(_translate("MainWindow", "Flat"))
        self.touchAdaptive.setToolTip(_translate("MainWindow", "Cursor travel distance depends on the mouse movement speed."))
        self.touchAdaptive.setText(_translate("MainWindow", "Adaptive"))
        self.touchpadRotationLabel.setText(_translate("MainWindow", "Rotation angle:"))
        self.touchRotationAngleSlider.setToolTip(_translate("MainWindow", "Sets the rotation angle of the device to the given angle, in degrees clockwise."))
        self.touchRotationAngle.setToolTip(_translate("MainWindow", "Sets the rotation angle of the device to the given angle, in degrees clockwise."))
        self.touchScrollingBox.setTitle(_translate("MainWindow", "Scrolling"))
        self.touchScrollFactor.setToolTip(_translate("MainWindow", "Scroll speed will be scaled by the given value."))
        self.label_6.setText(_translate("MainWindow", "Scrolling:"))
        self.touchNatScroll.setToolTip(_translate("MainWindow", "Touchscreen like scrolling."))
        self.touchNatScroll.setText(_translate("MainWindow", "Invert scroll direction"))
        self.label_12.setText(_translate("MainWindow", "Scrolling speed:"))
        self.label_11.setText(_translate("MainWindow", "Scrolling method:"))
        self.touchScrollNoScroll.setText(_translate("MainWindow", "No scroll"))
        self.touchScrollTwoFingers.setText(_translate("MainWindow", "Two fingers"))
        self.touchScrollEdges.setText(_translate("MainWindow", "Touchpad edges"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.TouchpadTab), _translate("MainWindow", "Touchpad"))
        self.TabletUseSettings.setText(_translate("MainWindow", "Use these settings"))
        self.tabletGeneralBox.setTitle(_translate("MainWindow", "General"))
        self.label_14.setText(_translate("MainWindow", "Tablet ID:"))
        self.toolMoveRelative.setText(_translate("MainWindow", "Relative"))
        self.label_15.setText(_translate("MainWindow", "Tool mode:"))
        self.tabletLeftHanded.setText(_translate("MainWindow", "Left handed mode"))
        self.label_16.setText(_translate("MainWindow", "Tool movement:"))
        self.toolMoveAbsolute.setText(_translate("MainWindow", "Absolute"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.TabletTab), _translate("MainWindow", "Tablet"))



# Form implementation generated from reading ui file 'sway_input_config/ui/about.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_about(object):
    def setupUi(self, about):
        about.setObjectName("about")
        about.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
        about.resize(450, 450)
        about.setMinimumSize(QtCore.QSize(450, 450))
        about.setMaximumSize(QtCore.QSize(450, 450))
        about.setModal(True)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(about)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.pixmap = QtWidgets.QLabel(parent=about)
        self.pixmap.setText("")
        self.pixmap.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.pixmap.setOpenExternalLinks(True)
        self.pixmap.setObjectName("pixmap")
        self.verticalLayout.addWidget(self.pixmap)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.label_2 = QtWidgets.QLabel(parent=about)
        self.label_2.setTextFormat(QtCore.Qt.TextFormat.PlainText)
        self.label_2.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.label_2.setWordWrap(True)
        self.label_2.setOpenExternalLinks(True)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.version = QtWidgets.QLabel(parent=about)
        self.version.setText("")
        self.version.setTextFormat(QtCore.Qt.TextFormat.AutoText)
        self.version.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.version.setWordWrap(True)
        self.version.setObjectName("version")
        self.verticalLayout.addWidget(self.version)
        self.label = QtWidgets.QLabel(parent=about)
        self.label.setText("<html><head/><body><p align=\"center\">Copyright: 2022-2023 Aleksey Samoilov</p><p align=\"center\">Licensed under the GNU GPLv3</p><p align=\"center\"><a href=\"https://github.com/Sunderland93/sway-input-config\"><span style=\" text-decoration: underline; color:#1397c3;\">Github</span></a></p></body></html>")
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=about)
        self.buttonBox.setWhatsThis("")
        self.buttonBox.setAccessibleName("")
        self.buttonBox.setAccessibleDescription("")
        self.buttonBox.setLocale(QtCore.QLocale(QtCore.QLocale.Language.English, QtCore.QLocale.Country.UnitedStates))
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.horizontalLayout.addWidget(self.buttonBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.retranslateUi(about)
        QtCore.QMetaObject.connectSlotsByName(about)

    def retranslateUi(self, about):
        _translate = QtCore.QCoreApplication.translate
        about.setWindowTitle(_translate("about", "About Sway Input Configurator"))
        self.label_2.setText(_translate("about", "A simple input device configurator for SwayWM, written in Python and Qt6. It uses standard libinput options to configure keyboard, touchpad and pointer devices"))



import json
import os
from i3ipc import Connection

def get_sway_version():
    i3 = Connection()
    sway_version = i3.get_version().human_readable

    return sway_version


def list_inputs_by_type(input_type=""):
    inputs = []

    i3 = Connection()
    all_inputs = i3.get_inputs()
    for i in all_inputs:
        if i.type == input_type or not input_type:
            inputs.append(i.identifier)

    return inputs


def get_config_home():
    config_home = os.getenv('XDG_CONFIG_HOME') if os.getenv('XDG_CONFIG_HOME') else os.path.join(
        os.getenv("HOME"), ".config/")
    return config_home


def get_data_dir():
    data_dir = ""
    home = os.getenv("HOME")
    xdg_data_home = os.getenv("XDG_DATA_HOME")

    if xdg_data_home:
        data_dir = os.path.join(xdg_data_home, "sway-input-config/")
    else:
        if home:
            data_dir = os.path.join(home, ".config/sway-input-config/")

    if not os.path.isdir(data_dir):
        print("Creating '{}'".format(data_dir))
        os.makedirs(data_dir, exist_ok=True)

    return data_dir


def load_text_file(path):
    try:
        with open(path, 'r') as file:
            data = file.read()
            return data
    except Exception as e:
        print(e)
        return None


def load_json(path):
    try:
        with open(path, 'r') as f:
            return json.load(f)
    except Exception as e:
        print("Error loading json: {}".format(e))
        return None


def save_json(src_dict, path):
    with open(path, 'w') as f:
        json.dump(src_dict, f, indent=2)


def save_list_to_text_file(data, file_path):
    text_file = open(file_path, "w")
    for line in data:
        text_file.write(line + "\n")
    text_file.close()


def reload_sway_config():
    i3 = Connection()
    i3.command('reload')



# Form implementation generated from reading ui file 'sway_input_config/ui/selectlayout.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_SelectKeyboardLayoutDialog(object):
    def setupUi(self, SelectKeyboardLayoutDialog):
        SelectKeyboardLayoutDialog.setObjectName("SelectKeyboardLayoutDialog")
        SelectKeyboardLayoutDialog.resize(520, 480)
        SelectKeyboardLayoutDialog.setMinimumSize(QtCore.QSize(520, 480))
        SelectKeyboardLayoutDialog.setMaximumSize(QtCore.QSize(520, 480))
        SelectKeyboardLayoutDialog.setLocale(QtCore.QLocale(QtCore.QLocale.Language.English, QtCore.QLocale.Country.UnitedStates))
        self.gridLayout = QtWidgets.QGridLayout(SelectKeyboardLayoutDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(parent=SelectKeyboardLayoutDialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(parent=SelectKeyboardLayoutDialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 2, 1, 1)
        self.layouts = QtWidgets.QListWidget(parent=SelectKeyboardLayoutDialog)
        self.layouts.setObjectName("layouts")
        self.gridLayout.addWidget(self.layouts, 1, 0, 1, 2)
        self.variants = QtWidgets.QListWidget(parent=SelectKeyboardLayoutDialog)
        self.variants.setObjectName("variants")
        self.gridLayout.addWidget(self.variants, 1, 2, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=SelectKeyboardLayoutDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 2, 0, 1, 3)

        self.retranslateUi(SelectKeyboardLayoutDialog)
        QtCore.QMetaObject.connectSlotsByName(SelectKeyboardLayoutDialog)

    def retranslateUi(self, SelectKeyboardLayoutDialog):
        _translate = QtCore.QCoreApplication.translate
        SelectKeyboardLayoutDialog.setWindowTitle(_translate("SelectKeyboardLayoutDialog", "Select a keyboard layout"))
        self.label.setText(_translate("SelectKeyboardLayoutDialog", "Language"))
        self.label_2.setText(_translate("SelectKeyboardLayoutDialog", "Variant"))
        self.layouts.setSortingEnabled(True)



from importlib.metadata import entry_points
from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='sway-input-config',
    version='1.4.1',
    description='input devices configurator for sway',
    license='GPL-3',
    author='Aleksey Samoilov',
    author_email='samoilov.lex@gmail.com',
    url='https://github.com/Sunderland93/sway-input-config',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    package_data={"": ["data/*", "ui/*", "langs/*"]},
    data_files=[
        ('share/applications', ['sway-input-config.desktop']),
        ('share/icons/hicolor/128x128/apps', ['sway-input-config.png']),
        ('share/metainfo', ['io.github.Sunderland93.sway-input-config.metainfo.xml']),
    ],
    install_requires=['PyQt6',
    'i3ipc',
    ],
    entry_points={
        'gui_scripts': [
            'sway-input-config = sway_input_config.main:main'
        ]
    }
)
